@@ -21,15 +21,15 @@
 #include <hildon/hildon.h>
 #include <libosso.h>
 #include "scout-window.h"
-#include "calendar-event-preview.hpp"
+#include "calendar-event-preview.h"
 #include <rtcom-eventlogger/event.h>
 #include <string.h>
 #include <libintl.h>
 #include <locale.h>
 //#include <CalInterface.h>
+#include <Common.h>
 #include <CMulticalendar.h>
 #include <CComponent.h>
-#include <Common.h>
 #include <vector>
 #include <ctime>
 #include <clockd/libtime.h>
@@ -41,13 +41,19 @@
 #include <telepathy-glib/dbus.h>
 #include <rtcom-telepathy-glib/extensions.h>
 #include <gconf/gconf-client.h>	
-
+#include "calendar-search-settings.h"
+#include "conversations-search-settings.h"
+#include "time-frame-selection.h"
+#include "calendar-event-editor.h"
+#include "calendar-journal-editor.h"
+#include "calendar-todo-editor.h"
 
 #define GC_ROOT "/apps/maemo/" PACKAGE 
 #define GCONF_SEARCH_CONTACTS GC_ROOT "/search_contacts"
 #define GCONF_SEARCH_CONVERSATIONS GC_ROOT "/search_conversations"
 #define GCONF_SEARCH_CALENDARS GC_ROOT "/search_calendars"
 #define GCONF_SEARCH_CASE_SENSITIVE GC_ROOT "/search_case_sensitive"
+
 struct _ScoutWindowPrivate
 {
   osso_context_t* osso;
@@ -81,6 +87,8 @@ struct _ScoutWindowPrivate
   gchar* current_search_text;
   GThread* search_thread;
   gboolean search_active;
+  calendar_search_settings_t* calendar_search_settings;
+  conversations_search_settings_t* conversations_search_settings;
 };
 
 #define SCOUT_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), SCOUT_TYPE_WINDOW, ScoutWindowPrivate))
@@ -140,7 +148,7 @@ _get_remote_name(ScoutWindow* self, const gchar* remote_uid)
   RTComElIter* iter = NULL;
   iter = rtcom_el_get_events(self->priv->rt_el, query);
   g_object_unref(query);
-  gboolean it = rtcom_el_iter_first(iter);
+  gboolean it = (iter != NULL) && rtcom_el_iter_first(iter);
   if(it)
   {
     gchar* remote_name = NULL;
@@ -165,7 +173,7 @@ _get_abook_uid_name(ScoutWindow* self, const gchar* remote_uid)
   RTComElIter* iter = NULL;
   iter = rtcom_el_get_events(self->priv->rt_el, query);
   g_object_unref(query);
-  gboolean it = rtcom_el_iter_first(iter);
+  gboolean it = (iter != NULL) && rtcom_el_iter_first(iter);
   if(it)
   {
     //    gchar* remote_name = NULL;
@@ -741,7 +749,7 @@ _search_in_events(ScoutWindow* self,
 {
   
   RTComElIter* rtcom_iter = rtcom_el_get_events(self->priv->rt_el, query);
-  gboolean it = rtcom_el_iter_first(rtcom_iter);
+  gboolean it = (rtcom_iter != NULL) &&rtcom_el_iter_first(rtcom_iter);
   while(self->priv->search_active && it)
   {
     gchar* free_text = NULL;
@@ -929,36 +937,139 @@ search_in_calendar(ScoutWindow* self)
   int error = 0;
   time_t start_time = -1;
   time_t end_time = 2145909599;
+  if(self->priv->calendar_search_settings->search_type == SEARCH_CUSTOM_INTERVAL)
+  {
+    start_time = self->priv->calendar_search_settings->start_time;
+    end_time = self->priv->calendar_search_settings->end_time;
+  }
+  else if(self->priv->calendar_search_settings->search_type == SEARCH_UPTO_TODAY)
+  {
+    start_time = self->priv->calendar_search_settings->start_time;
+  }
+  else if(self->priv->calendar_search_settings->search_type == SEARCH_THIS_YEAR)
+  {
+    struct tm now;
+    time_get_local(&now);
+    now.tm_sec = now.tm_min = now.tm_hour = 0;
+    now.tm_year--;
+    start_time = mktime(&now);
+  }
+  else if(self->priv->calendar_search_settings->search_type == SEARCH_THIS_MONTH)
+  {
+    struct tm now;
+    time_get_local(&now);
+    now.tm_sec = now.tm_min = now.tm_hour = 0;
+    now.tm_mon--;
+    start_time = mktime(&now);
+  }
+  else if(self->priv->calendar_search_settings->search_type == SEARCH_THIS_WEEK)
+  {
+    struct tm now;
+    time_get_local(&now);
+    now.tm_sec = now.tm_min = now.tm_hour = 0;
+    now.tm_mday-=7;
+    start_time = mktime(&now);
+  }
+  else if(self->priv->calendar_search_settings->search_type == SEARCH_THIS_DAY)
+  {
+    struct tm now;
+    time_get_local(&now);
+    now.tm_sec = now.tm_min = now.tm_hour = 0;
+    now.tm_mday--;
+    start_time = mktime(&now);
+  }
+  
   vector<CCalendar*> calendar_list = CMulticalendar::MCInstance()->getListCalFromMc();
   for(int j=0;self->priv->search_active && j<calendar_list.size();++j)
   {
-    int offset = 0;
     int limit = 100;
-    vector<CComponent*> component_list = calendar_list[j]->getAllEventsTodos(start_time,
-									     end_time,
-									     limit,
-									     offset,
-									     error);
-    while(component_list.size()>0)
+    vector<CComponent*> component_list;
+    if(self->priv->calendar_search_settings->search_events)
     {
-      offset += component_list.size();
-      for(int i = 0;i<component_list.size();++i)
+      int offset = 0;
+      component_list = calendar_list[j]->getComponents(1,
+						       start_time,
+						       end_time,
+						       limit,
+						       offset,
+						       error);
+      while(component_list.size()>0)
       {
-	_find_and_insert_calendar_event(self, component_list[i], calendar_list[j]->getCalendarId());
-	delete component_list[i];
+	offset += component_list.size();
+	for(int i = 0;i<component_list.size();++i)
+	{
+	  _find_and_insert_calendar_event(self, component_list[i], calendar_list[j]->getCalendarId());
+	  delete component_list[i];
+	}
+	component_list = calendar_list[j]->getComponents(1,
+							 start_time,
+							 end_time,
+							 limit,
+							 offset,
+							 error);
       }
-      component_list = calendar_list[j]->getAllEventsTodos(start_time,
+    }
+    if(self->priv->calendar_search_settings->search_tasks)
+    {
+
+      int offset = 0;
+      component_list = calendar_list[j]->getComponents(2,
+						       start_time,
+						       end_time,
+						       limit,
+						       offset,
+						       error);
+      while(component_list.size()>0)
+	{
+	  offset += component_list.size();
+	  for(int i = 0;i<component_list.size();++i)
+	    {
+	      _find_and_insert_calendar_event(self, component_list[i], calendar_list[j]->getCalendarId());
+	      delete component_list[i];
+	    }
+	  component_list = calendar_list[j]->getComponents(2,
+							   start_time,
 							   end_time,
 							   limit,
 							   offset,
 							   error);
+	}
     }
-
-    vector<CJournal*> journal_list = calendar_list[j]->getJournalListDetailed(error);
-    for(int i = 0;i<journal_list.size();++i)
+    if(self->priv->calendar_search_settings->search_birthdays)
     {
-      _find_and_insert_calendar_event(self, journal_list[i], calendar_list[j]->getCalendarId());
-      delete journal_list[i];
+      int offset = 0;
+      component_list = calendar_list[j]->getComponents(4,
+						       start_time,
+						       end_time,
+						       limit,
+						       offset,
+						       error);
+      while(component_list.size()>0)
+	{
+	  offset += component_list.size();
+	  for(int i = 0;i<component_list.size();++i)
+	    {
+	      _find_and_insert_calendar_event(self, component_list[i], calendar_list[j]->getCalendarId());
+	      delete component_list[i];
+	    }
+	  component_list = calendar_list[j]->getComponents(4,
+							   start_time,
+							   end_time,
+							   limit,
+							   offset,
+							   error);
+	}
+    }
+    
+    if(self->priv->calendar_search_settings->search_notes)
+    {
+
+      vector<CJournal*> journal_list = calendar_list[j]->getJournalListDetailed(error);
+      for(int i = 0;i<journal_list.size();++i)
+      {
+	_find_and_insert_calendar_event(self, journal_list[i], calendar_list[j]->getCalendarId());
+	delete journal_list[i];
+      }
     }
   }
   CMulticalendar::MCInstance()->releaseListCalendars(calendar_list);
@@ -1088,6 +1199,75 @@ _threaded_search_done(gpointer data)
   return FALSE;
 }
 
+void
+create_rtcom_query(ScoutWindow* self)
+{
+  if(self->priv->rt_com_chat_query)
+    g_object_unref(self->priv->rt_com_chat_query);
+  if(self->priv->rt_com_sms_query)
+    g_object_unref(self->priv->rt_com_sms_query);
+  self->priv->rt_com_chat_query = rtcom_el_query_new(self->priv->rt_el);
+  self->priv->rt_com_sms_query = rtcom_el_query_new(self->priv->rt_el);
+  if(self->priv->conversations_search_settings->search_type == SEARCH_ALL)
+  {
+    rtcom_el_query_prepare(self->priv->rt_com_chat_query, "service", "RTCOM_EL_SERVICE_CHAT", RTCOM_EL_OP_EQUAL, NULL);
+    rtcom_el_query_prepare(self->priv->rt_com_sms_query, "service", "RTCOM_EL_SERVICE_SMS", RTCOM_EL_OP_EQUAL, NULL);
+  }
+  else
+  {
+    if(self->priv->conversations_search_settings->search_type == SEARCH_CUSTOM_INTERVAL)
+    {
+      rtcom_el_query_prepare(self->priv->rt_com_chat_query, 
+			     "service", "RTCOM_EL_SERVICE_CHAT", RTCOM_EL_OP_EQUAL,
+			     "start-time", self->priv->conversations_search_settings->start_time , RTCOM_EL_OP_GREATER_EQUAL,
+			     "start-time", self->priv->conversations_search_settings->end_time , RTCOM_EL_OP_LESS_EQUAL,
+			     NULL);
+
+      rtcom_el_query_prepare(self->priv->rt_com_sms_query, 
+			     "service", "RTCOM_EL_SERVICE_SMS", RTCOM_EL_OP_EQUAL, 
+			     "start-time", self->priv->conversations_search_settings->start_time , RTCOM_EL_OP_GREATER_EQUAL,
+			     "start-time", self->priv->conversations_search_settings->end_time , RTCOM_EL_OP_LESS_EQUAL,
+			     NULL);
+    }
+    else
+    {
+      time_t start_time;
+      if(self->priv->conversations_search_settings->search_type == SEARCH_UPTO_TODAY)
+      {
+	start_time = self->priv->conversations_search_settings->start_time;
+      }
+      else
+      {
+	struct tm now;
+	time_get_local(&now);
+	now.tm_sec = now.tm_min = now.tm_hour = 0;
+	if(self->priv->conversations_search_settings->search_type == SEARCH_THIS_DAY)
+	{
+	  now.tm_mday--;
+	}
+	else if(self->priv->conversations_search_settings->search_type == SEARCH_THIS_MONTH)
+	{
+	  now.tm_mon--; 
+	}
+	else if(self->priv->conversations_search_settings->search_type == SEARCH_THIS_YEAR)
+	{
+	  now.tm_year--;
+	}
+	start_time = mktime(&now);
+      }
+      rtcom_el_query_prepare(self->priv->rt_com_chat_query, 
+			     "service", "RTCOM_EL_SERVICE_CHAT", RTCOM_EL_OP_EQUAL,
+			     "start-time", start_time , RTCOM_EL_OP_GREATER_EQUAL,
+			     NULL);
+
+      rtcom_el_query_prepare(self->priv->rt_com_sms_query, 
+			     "service", "RTCOM_EL_SERVICE_SMS", RTCOM_EL_OP_EQUAL, 
+			     "start-time", start_time , RTCOM_EL_OP_GREATER_EQUAL,
+			     NULL);
+    }
+  }
+}
+
 static gpointer
 _threaded_search_func(gpointer data)
 {
@@ -1097,12 +1277,21 @@ _threaded_search_func(gpointer data)
     return NULL;
   if(self->priv->search_in_contacts)
     search_in_contacts(self);
-  if(self->priv->search_in_conversations)
+  if(self->priv->search_in_conversations &&
+     (self->priv->conversations_search_settings->search_sms ||
+      self->priv->conversations_search_settings->search_im))
   {
-    search_in_conversations_sms(self);
-    search_in_conversations_chat(self);
+    create_rtcom_query(self);
+    if(self->priv->conversations_search_settings->search_sms)
+      search_in_conversations_sms(self);
+    if(self->priv->conversations_search_settings->search_im)
+      search_in_conversations_chat(self);
   }
-  if(self->priv->search_in_calendars)
+  if(self->priv->search_in_calendars &&
+     (self->priv->calendar_search_settings->search_events ||
+      self->priv->calendar_search_settings->search_tasks ||
+      self->priv->calendar_search_settings->search_notes ||
+      self->priv->calendar_search_settings->search_birthdays))
     search_in_calendar(self);
   g_idle_add((GSourceFunc)_threaded_search_done, self);
   return NULL;
@@ -1274,7 +1463,7 @@ scout_open_conversation_app(ScoutWindow* self,
   rtcom_el_query_prepare (query, "id", event_id, RTCOM_EL_OP_EQUAL, NULL);
   RTComElIter* iter = rtcom_el_get_events(self->priv->rt_el, query);
   g_object_unref(query);
-  if(rtcom_el_iter_first(iter))
+  if(iter && rtcom_el_iter_first(iter))
   {
     gchar* remote_uid = NULL;
     gchar* channel = NULL;
@@ -1376,7 +1565,7 @@ scout_open_event_log_window(ScoutWindow* self,
 			  "id", event_id, RTCOM_EL_OP_EQUAL,
 			  NULL);
   RTComElIter* rtcom_iter = rtcom_el_get_events(self->priv->rt_el, query);
-  if(rtcom_el_iter_first(rtcom_iter))
+  if(rtcom_iter && rtcom_el_iter_first(rtcom_iter))
   {
     int start_time = 0;
     int seven_days = 7 * 24 * 60 * 60;
@@ -1456,8 +1645,8 @@ scout_open_text_message_window(ScoutWindow* self,
   GtkWidget* text_view = hildon_text_view_new();
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_CHAR);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(text_view), FALSE);
-  gboolean it = rtcom_el_iter_first(iter);
-  if(it)
+
+  if(iter && rtcom_el_iter_first(iter))
   {
     gchar* text = NULL;
     rtcom_el_iter_get_values(iter, "free-text", &text, NULL);
@@ -1549,18 +1738,6 @@ open_result_from_list(GtkTreeView* tree_view,
 }
 
 
-static void
-_init_rtcom(ScoutWindow* self)
-{
-  self->priv->rt_el = rtcom_el_new();
-  self->priv->rt_com_chat_query = rtcom_el_query_new(self->priv->rt_el);
-  rtcom_el_query_prepare(self->priv->rt_com_chat_query, "service", "RTCOM_EL_SERVICE_CHAT", RTCOM_EL_OP_EQUAL, NULL);
-  self->priv->rt_com_sms_query = rtcom_el_query_new(self->priv->rt_el);
-  rtcom_el_query_prepare(self->priv->rt_com_sms_query, "service", "RTCOM_EL_SERVICE_SMS", RTCOM_EL_OP_EQUAL, NULL);
-}
-
-
-
 static void 
 _read_settings(ScoutWindow* self)
 {
@@ -1576,6 +1753,8 @@ _read_settings(ScoutWindow* self)
   self->priv->search_in_conversations = gconf_client_get_bool(client, GCONF_SEARCH_CONVERSATIONS , NULL);
   self->priv->search_in_calendars = gconf_client_get_bool(client, GCONF_SEARCH_CALENDARS , NULL);
   self->priv->search_case_sensitive = gconf_client_get_bool(client, GCONF_SEARCH_CASE_SENSITIVE, NULL);
+  calendar_search_settings_read(client, self->priv->calendar_search_settings);
+  conversations_search_settings_read(client, self->priv->conversations_search_settings);
   g_object_unref(client);
 }
 
@@ -1592,12 +1771,51 @@ _store_settings(ScoutWindow* self)
   g_object_unref(client);
 }
 
+static void
+toggle_contact_search_option(GtkToggleButton* button, gpointer user_data)
+{
+  g_return_if_fail(IS_SCOUT_WINDOW(user_data));
+  ScoutWindow* self = SCOUT_WINDOW(user_data);
+  self->priv->search_in_contacts = gtk_toggle_button_get_active(button);
+  g_free(self->priv->current_search_text);
+  self->priv->current_search_text = NULL;
+}
+
+static void
+toggle_conversation_search_option(GtkToggleButton* button, gpointer user_data)
+{
+  g_return_if_fail(IS_SCOUT_WINDOW(user_data));
+  ScoutWindow* self = SCOUT_WINDOW(user_data);
+  self->priv->search_in_conversations = gtk_toggle_button_get_active(button);
+  g_free(self->priv->current_search_text);
+  self->priv->current_search_text = NULL;
+}
+
+static void
+toggle_calendar_search_option(GtkToggleButton* button, gpointer user_data)
+{
+  g_return_if_fail(IS_SCOUT_WINDOW(user_data));
+  ScoutWindow* self = SCOUT_WINDOW(user_data);
+  self->priv->search_in_calendars = gtk_toggle_button_get_active(button);
+  g_free(self->priv->current_search_text);
+  self->priv->current_search_text = NULL;
+}
+
+static void
+toggle_case_sensitive_search_option(GtkToggleButton* button, gpointer user_data)
+{
+  g_return_if_fail(IS_SCOUT_WINDOW(user_data));
+  ScoutWindow* self = SCOUT_WINDOW(user_data);
+  self->priv->search_case_sensitive = gtk_toggle_button_get_active(button);
+  g_free(self->priv->current_search_text);
+  self->priv->current_search_text = NULL;
+}
+
 static void
 _show_settings_dialog_cb(GtkButton* button, gpointer data)
 {
   g_return_if_fail(IS_SCOUT_WINDOW(data));
   ScoutWindow* self = SCOUT_WINDOW(data);
-
   GtkWidget* dialog;
   dialog = gtk_dialog_new_with_buttons("Settings",
 				       GTK_WINDOW(self),
@@ -1606,29 +1824,34 @@ _show_settings_dialog_cb(GtkButton* button, gpointer data)
 				       GTK_RESPONSE_ACCEPT,
 				       NULL);
 
-  GtkWidget* button1 = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
-  GtkWidget* button2 = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
-  GtkWidget* button3 = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
-  GtkWidget* button4 = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
-
-  gtk_button_set_label(GTK_BUTTON(button1), dgettext("osso-addressbook", "addr_ap_address_book"));
-  gtk_button_set_label(GTK_BUTTON(button2), dgettext("rtcom-messaging-ui", "messaging_ap_conversations"));
-  gtk_button_set_label(GTK_BUTTON(button3), dgettext("calendar", "cal_ap_name"));
-  gtk_button_set_label(GTK_BUTTON(button4), "case sensitive");
+  GtkWidget* enable_contact_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* enable_conversation_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* enable_calendar_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* enable_case_sensitve_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  gtk_button_set_label(GTK_BUTTON(enable_contact_search), dgettext("osso-addressbook", "addr_ap_address_book"));
+  gtk_button_set_label(GTK_BUTTON(enable_conversation_search), dgettext("rtcom-messaging-ui", "messaging_ap_conversations"));
+  gtk_button_set_label(GTK_BUTTON(enable_calendar_search), dgettext("calendar", "cal_ap_name"));
+  gtk_button_set_label(GTK_BUTTON(enable_case_sensitve_search), gettext("case sensitive"));
   
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button1), FALSE);
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button2), FALSE);
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button3), FALSE);
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button4), FALSE);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(enable_contact_search), FALSE);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(enable_conversation_search), FALSE);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(enable_calendar_search), FALSE);
+  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(enable_case_sensitve_search), FALSE);
 
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button1), self->priv->search_in_contacts);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button2), self->priv->search_in_conversations);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button3), self->priv->search_in_calendars);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button4), self->priv->search_case_sensitive);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), button1, TRUE, TRUE, 3);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), button2, TRUE, TRUE, 3);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), button3, TRUE, TRUE, 3);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), button4, TRUE, TRUE, 3);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_contact_search), self->priv->search_in_contacts);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_conversation_search), self->priv->search_in_conversations);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_calendar_search), self->priv->search_in_calendars);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_case_sensitve_search), self->priv->search_case_sensitive);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), enable_contact_search, TRUE, TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), enable_conversation_search, TRUE, TRUE, 3);
+
+  GtkWidget* calendar_button_box = gtk_hbox_new(FALSE, 3);
+  GtkWidget* boxx = gtk_vbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(boxx), enable_calendar_search, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(calendar_button_box), boxx, TRUE, TRUE, 0);
+
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), calendar_button_box, TRUE, TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), enable_case_sensitve_search, TRUE, TRUE, 3);
 
   gtk_widget_show_all(dialog);
   int result = gtk_dialog_run(GTK_DIALOG(dialog));
@@ -1636,13 +1859,13 @@ _show_settings_dialog_cb(GtkButton* button, gpointer data)
   if(result = GTK_RESPONSE_ACCEPT)
   {
     self->priv->search_in_contacts = 
-      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button1));
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_contact_search));
     self->priv->search_in_conversations = 
-      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button2));
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_conversation_search));
     self->priv->search_in_calendars = 
-      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button3));
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_calendar_search));
     self->priv->search_case_sensitive = 
-      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button4));
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_case_sensitve_search));
   }
   gtk_widget_destroy(dialog);
 }
@@ -1652,10 +1875,41 @@ _init_menu(ScoutWindow* self)
 {
   HildonAppMenu* menu;
   menu = HILDON_APP_MENU(hildon_app_menu_new());
-  GtkWidget* settings_button = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
-  gtk_button_set_label(GTK_BUTTON(settings_button), "Settings");
-  g_signal_connect(settings_button, "clicked", G_CALLBACK(_show_settings_dialog_cb), self);
-  hildon_app_menu_append(menu, GTK_BUTTON(settings_button));
+
+  GtkWidget* case_sensitive_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* show_calendar_search_settings = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* show_conversations_search_settings = hildon_gtk_button_new(HILDON_SIZE_FINGER_HEIGHT);
+
+  GtkWidget* config_image_widget_cal = gtk_image_new_from_icon_name("general_settings",
+								    GtkIconSize(HILDON_ICON_SIZE_FINGER));
+  GtkWidget* config_image_widget_conv = gtk_image_new_from_icon_name("general_settings",
+								     GtkIconSize(HILDON_ICON_SIZE_FINGER));
+  GtkWidget* case_sensitive_icon = gtk_image_new_from_file("/usr/share/pixmaps/scout/case_sensitive.png");
+  
+  gtk_button_set_label(GTK_BUTTON(case_sensitive_search), gettext("case sensitive"));
+  gtk_button_set_label(GTK_BUTTON(show_calendar_search_settings), gettext("Calendar Search"));
+  gtk_button_set_label(GTK_BUTTON(show_conversations_search_settings), gettext("Conversations Search"));
+  gtk_button_set_image(GTK_BUTTON(case_sensitive_search), case_sensitive_icon);
+  gtk_button_set_image(GTK_BUTTON(show_calendar_search_settings), config_image_widget_cal);
+  gtk_button_set_image(GTK_BUTTON(show_conversations_search_settings), config_image_widget_conv);
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(case_sensitive_search), self->priv->search_case_sensitive);
+
+  g_signal_connect(show_calendar_search_settings, "clicked", 
+		   G_CALLBACK(show_calendar_search_settings_dlg), 
+		   self->priv->calendar_search_settings);
+  g_signal_connect(show_conversations_search_settings, "clicked", 
+		   G_CALLBACK(show_conversations_search_settings_dlg), 
+		   self->priv->conversations_search_settings);
+  g_signal_connect(case_sensitive_search, "toggled", 
+		   G_CALLBACK(toggle_case_sensitive_search_option), 
+		   self);
+
+  hildon_app_menu_append(menu, GTK_BUTTON(show_calendar_search_settings));
+  hildon_app_menu_append(menu, GTK_BUTTON(show_conversations_search_settings));
+  hildon_app_menu_append(menu, GTK_BUTTON(case_sensitive_search));
+
+
   gtk_widget_show_all(GTK_WIDGET(menu));
   hildon_window_set_app_menu(HILDON_WINDOW(self), menu);
 }
@@ -1664,14 +1918,12 @@ static void
 scout_window_init(ScoutWindow* self)
 {
   self->priv = SCOUT_WINDOW_GET_PRIVATE(self);
-  _init_rtcom(self);
   self->priv->cached_calendar_id_icon = 
     g_hash_table_new_full(g_direct_hash, g_direct_equal,
 			  NULL, (GDestroyNotify) g_object_unref);
   self->priv->cached_calendar_type_icon = 
     g_hash_table_new_full(g_direct_hash, g_direct_equal,
 			  NULL, (GDestroyNotify) g_object_unref);
-
   self->priv->cached_contact_attribute_icons = 
     g_hash_table_new_full(g_str_hash, g_str_equal,
 			  (GDestroyNotify) g_free, (GDestroyNotify) g_object_unref);
@@ -1701,7 +1953,6 @@ scout_window_init(ScoutWindow* self)
 			     "general_sms",
 			     HILDON_ICON_PIXEL_SIZE_FINGER, 
 			     GTK_ICON_LOOKUP_NO_SVG, NULL);
-
   self->priv->search_category_conversation_outgoing_sms_icon = 
     gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
 			     "chat_replied_sms",
@@ -1728,7 +1979,33 @@ scout_window_init(ScoutWindow* self)
   self->priv->search_in_calendars = TRUE;
   self->priv->search_case_sensitive = FALSE;
 
-  GtkWidget* box = gtk_vbox_new(FALSE, 0);
+  GtkWidget* contact_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* conversation_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+  GtkWidget* calendar_search = hildon_gtk_toggle_button_new(HILDON_SIZE_FINGER_HEIGHT);
+
+  g_signal_connect(contact_search, "toggled", G_CALLBACK(toggle_contact_search_option), self);
+  g_signal_connect(conversation_search, "toggled", G_CALLBACK(toggle_conversation_search_option), self);
+  g_signal_connect(calendar_search, "toggled", G_CALLBACK(toggle_calendar_search_option), self);
+  GtkWidget* contacts_icon = gtk_image_new_from_icon_name("general_contacts",
+							  HILDON_ICON_SIZE_FINGER);
+  GtkWidget* calendar_icon = gtk_image_new_from_icon_name("general_calendar",
+							  HILDON_ICON_SIZE_FINGER);
+  GtkWidget* conversation_icon = gtk_image_new_from_icon_name("general_chat",
+							      HILDON_ICON_SIZE_FINGER);
+
+  
+  gtk_button_set_image(GTK_BUTTON(contact_search),
+		       contacts_icon);
+  gtk_button_set_image(GTK_BUTTON(calendar_search),
+		       calendar_icon);
+  gtk_button_set_image(GTK_BUTTON(conversation_search),
+		       conversation_icon);
+
+  GtkWidget* content_box = gtk_vbox_new(FALSE, 0);
+  GtkWidget* category_selection_box = gtk_vbox_new(FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(category_selection_box), contact_search, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(category_selection_box), conversation_search, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(category_selection_box), calendar_search, FALSE, FALSE, 0);
   self->priv->search_entry_field = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT);
   g_object_set(self->priv->search_entry_field,"hildon-input-mode",
                HILDON_GTK_INPUT_MODE_FULL, NULL);
@@ -1748,7 +2025,6 @@ scout_window_init(ScoutWindow* self)
   GtkCellRenderer* renderer4 = NULL;
   self->priv->search_touch_list = hildon_gtk_tree_view_new(HILDON_UI_MODE_NORMAL);
 
-
   self->priv->search_button = hildon_button_new_with_text(HILDON_SIZE_FINGER_HEIGHT,
 							  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
 							  dgettext("hildon-libs", "wdgt_bd_search"), NULL);
@@ -1762,7 +2038,7 @@ scout_window_init(ScoutWindow* self)
   gtk_tree_view_column_set_sizing (column2,
 				   GTK_TREE_VIEW_COLUMN_FIXED);
   gtk_tree_view_column_set_fixed_width (column2,
-					785 - (48+48+48));
+					785 - (48+48+48+48+48));
 
   GtkTreeViewColumn* column3 = gtk_tree_view_column_new();
   GtkTreeViewColumn* column4 = gtk_tree_view_column_new();
@@ -1803,28 +2079,40 @@ scout_window_init(ScoutWindow* self)
 
   GtkWidget* search_touch_alignment = gtk_alignment_new(0, 0, 1.0f, 1.0f);
 
-  GtkWidget* search_box = gtk_hbox_new(FALSE, 3);
-
   gtk_container_add(GTK_CONTAINER(search_touch_alignment), pan);
-
-
   gtk_container_add(GTK_CONTAINER(pan), self->priv->search_touch_list);
 
   g_signal_connect(self->priv->search_entry_field, "activate", G_CALLBACK(on_search_enter), self);
   g_signal_connect(self->priv->search_button, "clicked", G_CALLBACK(on_search_clicked), self);
   g_signal_connect(self->priv->search_touch_list, "row-activated", G_CALLBACK(open_result_from_list), self);
 
-  gtk_box_pack_start(GTK_BOX(search_box), self->priv->search_entry_field, TRUE, TRUE, 0);
-  gtk_box_pack_end(GTK_BOX(search_box), self->priv->search_button, FALSE, FALSE, 0);
 
+
+  GtkWidget* search_box = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(search_box), self->priv->search_entry_field, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(search_box), self->priv->search_button, FALSE, FALSE, 0);
   gtk_container_add(GTK_CONTAINER(entry_alignment), search_box);
-  gtk_box_pack_start(GTK_BOX(box), search_touch_alignment, TRUE, TRUE, 0);
-  gtk_box_pack_end(GTK_BOX(box), entry_alignment, FALSE, FALSE, 0);
-  gtk_widget_show_all(box);
-  gtk_container_add(GTK_CONTAINER(self), box);
+  GtkWidget* search_result_and_category_box = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(search_result_and_category_box), search_touch_alignment, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(search_result_and_category_box), category_selection_box, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(content_box), search_result_and_category_box, TRUE, TRUE, 0);
+  gtk_box_pack_end(GTK_BOX(content_box), entry_alignment, FALSE, FALSE, 0);
+
+
+  gtk_widget_show_all(content_box);
+  gtk_container_add(GTK_CONTAINER(self), content_box);
   gtk_widget_grab_focus(self->priv->search_entry_field);
+  self->priv->calendar_search_settings = g_new0(calendar_search_settings_t, 1);
+  self->priv->conversations_search_settings = g_new0(conversations_search_settings_t, 1);
   _read_settings(self);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(contact_search), self->priv->search_in_contacts);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(conversation_search), self->priv->search_in_conversations);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(calendar_search), self->priv->search_in_calendars);
   _init_menu(self);
+  self->priv->rt_el = rtcom_el_new();
+  self->priv->rt_com_chat_query = NULL;
+  self->priv->rt_com_sms_query = NULL;
+  create_rtcom_query(self);
 }
 
 static void
@@ -1857,6 +2145,8 @@ scout_window_finalize(GObject* object)
   g_hash_table_destroy(self->priv->cached_calendar_type_icon);
   g_hash_table_destroy(self->priv->cached_account_icon);
   _store_settings(SCOUT_WINDOW(object));
+  g_free(self->priv->calendar_search_settings);
+  g_free(self->priv->conversations_search_settings);
   G_OBJECT_CLASS(scout_window_parent_class)->finalize(object);
 }
 
